﻿using System;
using System.ComponentModel;
using System.Net;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// 네트워크 연결
    /// </summary>
    public class NetworkConnection : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region WNET 연결 추가하기 2 - WNetAddConnection2(networkResource, password, userName, flag)

        /// <summary>
        /// WNET 연결 추가하기 2
        /// </summary>
        /// <param name="networkResource">네트워크 리소스</param>
        /// <param name="password">패스워드</param>
        /// <param name="userName">사용자명</param>
        /// <param name="flag">플래그</param>
        /// <returns>처리 결과</returns>
        [DllImport("mpr")]
        private static extern int WNetAddConnection2(NetworkResource networkResource, string password, string userName, int flag);

        #endregion
        #region WNET 연결 취소하기 2 - WNetCancelConnection2(name, flag, force)

        /// <summary>
        /// WNET 연결 취소하기 2
        /// </summary>
        /// <param name="name">네트워크명</param>
        /// <param name="flag">플래그</param>
        /// <param name="force">강제 여부</param>
        /// <returns>처리 결과</returns>
        [DllImport("mpr")]
        private static extern int WNetCancelConnection2(string name, int flag, bool force);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 로컬명
        /// </summary>
        private string localName;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - NetworkConnection(localName, remoteName, credential)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="localName">로컬명</param>
        /// <param name="remoteName">원격명</param>
        /// <param name="credential">자격 증명</param>
        public NetworkConnection(string localName, string remoteName, NetworkCredential credential)
        {
            this.localName = localName;

            NetworkResource networkResource = new NetworkResource()
            {
                Scope        = ResourceScope.GlobalNetwork,
                ResourceType = ResourceType.Disk,
                DisplayType  = ResourceDisplayType.Share,
                LocalName    = localName,
                RemoteName   = remoteName
            };

            string userName;
            
            if(string.IsNullOrEmpty(credential.Domain))
            {
                userName = credential.UserName;
            }
            else
            {
                userName = string.Format(@"{0}\{1}", credential.Domain, credential.UserName);
            }

            int result = WNetAddConnection2(networkResource, credential.Password, userName, 0);

            if(result != 0)
            {
                throw new Win32Exception(result);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Destructor

        #region 소멸자 - ~NetworkConnection()

        /// <summary>
        /// 소멸자
        /// </summary>
        ~NetworkConnection()
        {
            Dispose(false);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            Dispose(true);

            GC.SuppressFinalize(this);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 리소스 해제하기 - Dispose(disposing)

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        /// <param name="disposing">해제 여부</param>
        protected virtual void Dispose(bool disposing)
        {
            WNetCancelConnection2(this.localName, 0, true);
        }

        #endregion
    }
}